# -*- coding: utf8 -*-
import math
import select
from zlib import adler32
from time import gmtime, strftime, time

from flask import Flask, Response, request, render_template, jsonify, redirect, url_for, abort, flash, send_file
from flask.ext.login import login_user, login_required, logout_user 
from werkzeug.datastructures import Headers

from burpui import app, bui, login_manager
from burpui.forms import LoginForm
from burpui.misc.utils import human_readable as _hr
from burpui.misc.backend.interface import BUIserverException

@login_manager.user_loader
def load_user(userid):
    if bui.auth != 'none':
        return bui.uhandler.user(userid)
    return None

@app.route('/settings', methods=['GET', 'POST'])
@app.route('/<server>/settings', methods=['GET', 'POST'])
@login_required
def settings(server=None):
    if request.method == 'POST':
        noti = bui.cli.store_conf(request.form)
        return jsonify(notif=noti)
    return render_template('settings.html', settings=True, server=server)

@app.route('/api/server-config')
@app.route('/api/<server>/server-config')
@login_required
def read_conf_srv(server=None):
    r = bui.cli.read_conf(server)
    return jsonify(results=r,
                   boolean=bui.cli.get_parser_attr('boolean'),
                   string=bui.cli.get_parser_attr('string'),
                   integer=bui.cli.get_parser_attr('integer'),
                   multi=bui.cli.get_parser_attr('multi'),
                   server_doc=bui.cli.get_parser_attr('server_doc'),
                   suggest=bui.cli.get_parser_attr('values_server'),
                   placeholders=bui.cli.get_parser_attr('placeholders'),
                   defaults=bui.cli.get_parser_attr('defaults_server'))


"""
Here is the API

The whole API returns JSON-formated data
"""

@app.route('/api/restore/<name>/<int:backup>', methods=['POST'])
@app.route('/api/<server>/restore/<name>/<int:backup>', methods=['POST'])
@login_required
def restore(server=None, name=None, backup=None):
    l = request.form.get('list')
    s = request.form.get('strip')
    resp = None
    if not l or not name or not backup:
        abort(500)
    if server:
        filename = 'restoration_%d_%s_on_%s_at_%s.zip' % (backup, name, server, strftime("%Y-%m-%d_%H_%M_%S", gmtime()))
    else:
        filename = 'restoration_%d_%s_at_%s.zip' % (backup, name, strftime("%Y-%m-%d_%H_%M_%S", gmtime()))
    if not server:
        archive = bui.cli.restore_files(name, backup, l, s)
        if not archive:
            abort(500)
        try:
            resp = send_file(archive, as_attachment=True, attachment_filename=filename, mimetype='application/zip')
            resp.set_cookie('fileDownload', 'true')
        except Exception, e:
            app.logger.error(str(e))
            abort(500)
    else:
        socket = None
        try:
            socket, length = bui.cli.restore_files(name, backup, l, s, server)
            app.logger.debug('Need to get %d Bytes : %s', length, socket)
            def stream_file(sock, l):
                bsize = 1024
                received = 0
                if l < bsize:
                    bsize = l
                while received < l:
                    buf = b''
                    r, _, _ = select.select([sock], [], [], 5)
                    if not r:
                        raise Exception ('Socket timed-out')
                    buf += sock.recv(bsize)
                    if not buf:
                        continue
                    received += len(buf)
                    app.logger.debug('%d/%d', received, l)
                    yield buf
                sock.close()

            headers = Headers()
            headers.add('Content-Disposition', 'attachment', filename=filename)
            headers['Content-Length'] = length

            resp = Response(stream_file(socket, length), mimetype='application/zip',
                            headers=headers, direct_passthrough=True)
            resp.set_cookie('fileDownload', 'true')
            resp.set_etag('flask-%s-%s-%s' % (
                    time(),
                    length,
                    adler32(filename.encode('utf-8')) & 0xffffffff))
        except Exception, e:
            app.logger.error(str(e))
            abort(500)
    return resp

@app.route('/api/running-clients.json')
@app.route('/api/<server>/running-clients.json')
@login_required
def running_clients(server=None):
    """
    API: running_clients
    :returns: a list of running clients
    """
    if not server:
        server = request.args.get('server')
    r = bui.cli.is_one_backup_running(server)
    return jsonify(results=r)

@app.route('/api/render-live-template', methods=['GET'])
@app.route('/api/<server>/render-live-template', methods=['GET'])
@app.route('/api/render-live-template/<name>')
@app.route('/api/<server>/render-live-template/<name>')
@login_required
def render_live_tpl(server=None, name=None):
    """
    API: render_live_tpl
    :param name: the client name if any. You can also use the GET parameter
    'name' to achieve the same thing
    :returns: HTML that should be included directly into the page
    """
    if not server:
        server = request.args.get('server')
    if not name:
        name = request.args.get('name')
    if not name:
        abort(500)
    if isinstance(bui.cli.running, dict):
        if server and name not in bui.cli.running[server]:
            abort(404)
        else:
            found = False
            for k, a in bui.cli.running.iteritems():
                found = found or (name in a)
            if not found:
                abort(404)
    else:
        if name not in bui.cli.running:
            abort(404)
    try:
        counters = bui.cli.get_counters(name, agent=server)
    except BUIserverException:
        counters = []
    return render_template('live-monitor-template.html', cname=name, counters=counters, server=server)

@app.route('/api/servers.json')
@login_required
def servers_json():
    r = []
    if hasattr(bui.cli, 'servers'):
        for serv in bui.cli.servers:
            r.append({'name': serv, 'clients': len(bui.cli.servers[serv].get_all_clients(serv)), 'alive': bui.cli.servers[serv].ping()})
    return jsonify(results=r)

@app.route('/api/live.json')
@app.route('/api/<server>/live.json')
@login_required
def live(server=None):
    """
    API: live
    :returns: the live status of the server
    """
    if not server:
        server = request.args.get('server')
    r = []
    if server:
        l = (bui.cli.is_one_backup_running(server))[server]
    else:
        l = bui.cli.is_one_backup_running()
    if isinstance(l, dict):
        for k, a in l.iteritems():
            for c in a:
                s = {}
                s['client'] = c
                s['agent'] = k
                try:
                    s['status'] = bui.cli.get_counters(c, agent=k)
                except BUIserverException:
                    s['status'] = []
                r.append(s)
    else:
        for c in l:
            s = {}
            s['client'] = c
            try:
                s['status'] = bui.cli.get_counters(c, agent=server)
            except BUIserverException:
                s['status'] = []
            r.append(s)
    return jsonify(results=r)

@app.route('/api/running.json')
@app.route('/api/<server>/running.json')
@login_required
def backup_running(server=None):
    """
    API: backup_running
    :returns: true if at least one backup is running
    """
    j = bui.cli.is_one_backup_running(server)
    r = False
    if isinstance(j, dict):
        for k, v in j.iteritems():
            r = r or (len(v) > 0)
    else:
        r = len(j) > 0
    return jsonify(results=r)

@app.route('/api/client-tree.json/<name>/<int:backup>', methods=['GET'])
@app.route('/api/<server>/client-tree.json/<name>/<int:backup>', methods=['GET'])
@login_required
def client_tree(server=None, name=None, backup=None):
    """
    WebService: return a specific client files tree
    :param name: the client name (mandatory)
    :param backup: the backup number (mandatory)

    """
    if not server:
        server = request.args.get('server')
    j = []
    if not name or not backup:
        return jsonify(results=j)
    root = request.args.get('root')
    try:
        j = bui.cli.get_tree(name, backup, root, agent=server)
    except BUIserverException, e:
        err = [[2, str(e)]]
        return jsonify(notif=err)
    return jsonify(results=j)

@app.route('/api/clients-report.json')
@app.route('/api/<server>/clients-report.json')
@login_required
def clients_report_json(server=None):
    """
    WebService: return a JSON with global stats
    """
    if not server:
        server = request.args.get('server')
    j = []
    try:
        clients = bui.cli.get_all_clients(agent=server)
    except BUIserverException, e:
        err = [[2, str(e)]]
        return jsonify(notif=err)
    cl = []
    ba = []
    for c in clients:
        client = bui.cli.get_client(c['name'], agent=server)
        if not client:
            continue
        cl.append( { 'name': c['name'], 'stats': bui.cli.get_backup_logs(client[-1]['number'], c['name'], agent=server) } )
        for b in client:
            ba.append(bui.cli.get_backup_logs(b['number'], c['name'], True, agent=server))
    j.append( { 'clients': cl, 'backups': sorted(ba, key=lambda k: k['end']) } )
    return jsonify(results=j)

@app.route('/api/client-stat.json/<name>')
@app.route('/api/<server>/client-stat.json/<name>')
@app.route('/api/client-stat.json/<name>/<int:backup>')
@app.route('/api/<server>/client-stat.json/<name>/<int:backup>')
@login_required
def client_stat_json(server=None, name=None, backup=None):
    """
    WebService: return a specific client detailed report
    """
    if not server:
        server = request.args.get('server')
    j = []
    if not name:
        err = [[1, 'No client defined']]
        return jsonify(notif=err)
    if backup:
        try:
            j = bui.cli.get_backup_logs(backup, name, agent=server)
        except BUIserverException, e:
            err = [[2, str(e)]]
            return jsonify(notif=err)
    else:
        try:
            cl = bui.cli.get_client(name, agent=server)
        except BUIserverException, e:
            err = [[2, str(e)]]
            return jsonify(notif=err)
        for c in cl:
            j.append(bui.cli.get_backup_logs(c['number'], name, agent=server))
    return jsonify(results=j)

@app.route('/api/client.json/<name>')
@app.route('/api/<server>/client.json/<name>')
@login_required
def client_json(server=None, name=None):
    """
    WebService: return a specific client backups overview
    """
    if not server:
        server = request.args.get('server')
    try:
        j = bui.cli.get_client(name, agent=server)
    except BUIserverException, e:
        err = [[2, str(e)]]
        return jsonify(notif=err)
    return jsonify(results=j)

@app.route('/api/clients.json')
@app.route('/api/<server>/clients.json')
@login_required
def clients_json(server=None):
    """
    WebService: return a JSON listing all clients
    """
    if not server:
        server = request.args.get('server')
    try:
        j = bui.cli.get_all_clients(agent=server)
    except BUIserverException, e:
        err = [[2, str(e)]]
        return jsonify(notif=err)
    return jsonify(results=j)

"""
Here are some custom filters
"""
@app.template_filter()
def mypad (s):
    """
    Filter: used to pad 0's to backup numbers as in the burp's status monitor
    """
    if not s:
        return '0000000'
    return '{0:07d}'.format(int(s))

@app.template_filter()
def time_human(d):
    s = ''
    seconds = (((d % 31536000) % 86400) % 3600) % 60
    minutes = math.floor((((d % 31536000) % 86400) % 3600) / 60)
    hours   = math.floor(((d % 31536000) % 86400) / 3600)
    if hours > 0:
        s = '%02dH' % hours
    return '%s %02dm %02ds' % (s, minutes, seconds)

@app.template_filter()
def bytes_human(b):
    return '{0:.1eM}'.format(_hr(b))

"""
And here is the main site
"""

@app.route('/live-monitor')
@app.route('/<server>/live-monitor')
@app.route('/live-monitor/<name>')
@app.route('/<server>/live-monitor/<name>')
@login_required
def live_monitor(server=None, name=None):
    """
    Live status monitor view
    """
    if not server:
        server = request.args.get('server')
    if bui.standalone:
        if not bui.cli.running:
            flash('Sorry, there are no running backups', 'warning')
            return redirect(url_for('home'))
    else:
        run = False
        for a in bui.cli.servers:
            run = run or (a in bui.cli.running and bui.cli.running[a])
        if not run:
            flash('Sorry, there are no running backups', 'warning')
            return redirect(url_for('home'))
            
    return render_template('live-monitor.html', live=True, cname=name, server=server)

@app.route('/client-browse/<name>', methods=['GET'])
@app.route('/<server>/client-browse/<name>', methods=['GET'])
@app.route('/client-browse/<name>/<int:backup>')
@app.route('/<server>/client-browse/<name>/<int:backup>')
@login_required
def client_browse(server=None, name=None, backup=None):
    """
    Browse a specific backup of a specific client
    """
    if not server:
        server = request.args.get('server')
    bkp = request.args.get('backup')
    if bkp and not backup:
        return redirect(url_for('client_browse', name=name, backup=bkp, server=server))
    return render_template('client-browse.html', tree=True, backup=True, overview=True, cname=name, nbackup=backup, server=server)

@app.route('/client-report/<name>')
@app.route('/<server>/client-report/<name>')
@login_required
def client_report(server=None, name=None):
    """
    Specific client report
    """
    if not server:
        server = request.args.get('server')
    l = bui.cli.get_client(name, agent=server)
    if len(l) == 1:
        return redirect(url_for('backup_report', name=name, backup=l[0]['number'], server=server))
    return render_template('client-report.html', client=True, report=True, cname=name, server=server)

@app.route('/clients-report')
@app.route('/<server>/clients-report')
@login_required
def clients_report(server=None):
    """
    Global report
    """
    if not server:
        server = request.args.get('server')
    return render_template('clients-report.html', clients=True, report=True, server=server)

@app.route('/backup-report/<name>', methods=['GET'])
@app.route('/<server>/backup-report/<name>', methods=['GET'])
@app.route('/backup-report/<name>/<int:backup>', methods=['GET'])
@app.route('/<server>/backup-report/<name>/<int:backup>', methods=['GET'])
@login_required
def backup_report(server=None, name=None, backup=None):
    """
    Backup specific report
    """
    if not backup:
        backup = request.args.get('backup')
    if not server:
        server = request.args.get('server')
    return render_template('backup-report.html', client=True, backup=True, report=True, cname=name, nbackup=backup, server=server)

@app.route('/client', methods=['GET'])
@app.route('/<server>/client', methods=['GET'])
@app.route('/client/<name>')
@app.route('/<server>/client/<name>')
@login_required
def client(server=None, name=None):
    """
    Specific client overview
    """
    if name:
        c = name
    else:
        c = request.args.get('name')
    if not server:
        server = request.args.get('server')
    if bui.cli.is_backup_running(c, agent=server):
        return redirect(url_for('live_monitor', name=name, server=server))
    return render_template('client.html', client=True, overview=True, cname=c, server=server)

@app.route('/clients', methods=['GET'])
@app.route('/<server>/clients', methods=['GET'])
@login_required
def clients(server=None):
    if not server:
        server = request.args.get('server')
    return render_template('clients.html', clients=True, overview=True, server=server)

@app.route('/servers', methods=['GET'])
@login_required
def servers():
    return render_template('servers.html', servers=True, overview=True)

@app.route('/login', methods=['POST', 'GET'])
def login():
    form = LoginForm(request.form)
    if form.validate_on_submit():
        user = bui.uhandler.user(form.username.data)
        if user.active and user.login(form.username.data, passwd=form.password.data):
            login_user(user, remember=form.remember.data)
            flash('Logged in successfully', 'success')
            return redirect(request.args.get("next") or url_for('home'))
        else:
            flash('Wrong username or password', 'danger')
    return render_template('login.html', form=form, login=True)

@app.route('/logout')
@login_required
def logout():
    logout_user()
    return redirect(url_for('home'))

@app.route('/')
@login_required
def home():
    """
    Home page
    """
    if bui.standalone:
        return redirect(url_for('clients'))
    else:
        return redirect(url_for('servers'))



# -*- coding: utf8 -*-
"""
Burp-UI is a web-ui for burp backup written in python with Flask and
jQuery/Bootstrap
"""
__title__        = 'burp-ui'
__author__       = 'Benjamin SANS (Ziirish)'
__author_email__ = 'ziirish+burpui@ziirish.info'
__url__          = 'https://git.ziirish.me/ziirish/burp-ui'
__description__  = 'Burp-UI is a web-ui for burp backup written in python with Flask and jQuery/Bootstrap'
__license__      = 'BSD 3-clause'
__version__      = '0.0.6'

import os

from flask import Flask
from flask.ext.login import LoginManager
from burpui.server import BUIServer as BurpUI

# First, we setup the app
app = Flask(__name__)

app.config['CFG'] = None

app.secret_key = 'VpgOXNXAgcO81xFPyWj07ppN6kExNZeCDRShseNzFKV7ZCgmW2/eLn6xSlt7pYAVBj12zx2Vv9Kw3Q3jd1266A=='
app.jinja_env.globals.update(isinstance=isinstance,list=list)

# We initialize the core
bui = BurpUI(app)

# And the login_manager
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = 'login'
login_manager.login_message_category = 'info'

# Then we load our routes
import burpui.routes

def init(conf=None, debug=False, gunicorn=True):
    app.config['DEBUG'] = debug
    if debug:
        app.config['TESTING'] = True

    if conf:
        if os.path.isfile(conf):
            app.config['CFG'] = conf
        else:
            raise IOError('File not found: \'{0}\''.format(conf))
    else:
        conf_files = ['/etc/burp/burpui.cfg', os.path.join(os.path.dirname(os.path.realpath(__file__)), '..', '..', '..', 'share', 'burpui', 'etc', 'burpui.cfg')]
        for p in conf_files:
            app.logger.debug('Trying file \'%s\'', p)
            if os.path.isfile(p):
                app.config['CFG'] = p
                app.logger.debug('Using file \'%s\'', p)
                break

    bui.setup(app.config['CFG'])

    if gunicorn:
        from werkzeug.contrib.fixers import ProxyFix
        app.wsgi_app = ProxyFix(app.wsgi_app)

    return app



# -*- coding: utf8 -*-
import re
import os
import shutil
import codecs

from burpui.misc.utils import human_readable as _hr, BUIlogging
from burpui.misc.parser.interface import BUIparser

class Parser(BUIparser,BUIlogging):
    defaults_server = {
            u'mode': '',
            u'address': '', #IP
            u'port': 4971, #int
            u'status_port': 4972, #int
            u'status_address': '', #127.0.0.1 / ::1
            u'daemon': True, #bool
            u'fork': True, #bool
            u'directory': '',
            u'directory_tree': True, #bool
            u'timestamp_format': '',
            u'password_check': True, #bool
            u'clientconfdir': '', #dir
            u'pidfile': '',
            u'syslog': False, #bool
            u'stdout': True, #bool
            u'keep': [7, 6, 4], #multi #int
            u'hardlinked_archive': False, #bool
            u'max_hardlinks': 10000, #int
            u'librsync': True, #bool
            u'compression': 'gzip9',
            u'version_warn': True, #bool
            u'path_length_warn': True, #bool
            u'protocol': False, #int
            u'client_lockdir': '',
            u'user': '',
            u'group': '',
            u'umask': 0022, #mode
            u'ratelimit': False, #int
            u'network_timeout': 7200, #int
            u'working_dir_recovery_method': '',
            u'client_can_delete': True, #bool
            u'client_can_diff': True, #bool
            u'client_can_force_backup': True, #bool
            u'client_can_list': True, #bool
            u'client_can_restore': True, #bool
            u'client_can_verify': True, #bool
            u'restore_client': '', #multi
            u'ssl_cert_ca': '', #file
            u'ssl_cert': '', #file
            u'ssl_key': '', #file
            u'ssl_key_password': '',
            u'ssl_ciphers': '',
            u'ssl_compression': 'zlib5',
            u'ssl_dhfile': '', #file
            u'max_children': 5, #int
            u'max_status_children': 5, #int
            u'max_storage_subdirs': 30000, #int
            u'timer_script': '', #file
            u'timer_arg': '', #multi
            u'notify_success_script': '', #file
            u'notify_success_arg': '', #multi
            u'notify_success_warnings_only': False, #bool
            u'notify_success_changes_only': False, #bool
            u'notify_failure_script': '', #file
            u'notify_failure_arg': '', #multi
            u'dedup_group': '',
            u'server_script_pre': '', #file
            u'server_script_pre_arg': '', #multi
            u'server_script_pre_notify': False, #bool
            u'server_script_post': '', #file
            u'server_script_post_arg': '', #multi
            u'server_script_post_notify': False, #bool
            u'server_script': '', #file
            u'server_script_arg': '', #multi
            u'server_script_notify': False, #bool
            u'server_script_post_run_on_fail': False, #bool
            u'autoupgrade_dir': '', #dir
            u'ca_conf': '', #file
            u'ca_name': '',
            u'ca_server_name': '',
            u'ca_burp_ca': '', #file
            u'monitor_browse_cache': False, #bool
            u'hard_quota': '',
            u'soft_quota': '',
        }
    placeholders = {
            u'port': "port number",
            u'status_port': "port number",
            u'daemon': "0|1",
            u'fork': "0|1",
            u'directory': "path",
            u'directory_tree': "0|1",
            u'timestamp_format': "strftime format",
            u'password_check': "0|1",
            u'manual_delete': "path",
            u'clientconfdir': "path",
            u'lockfile': "path",
            u'pidfile': "path",
            u'syslog': "0|1",
            u'stdout': "0|1",
            u'keep': "number",
            u'hardlinked_archive': "0|1",
            u'max_hardlinks': "number",
            u'librsync': "0|1",
            u'compression': "gzip[0-9]",
            u'hard_quota': "b/Kb/Mb/Gb",
            u'soft_quota': "b/Kb/Mb/Gb",
            u'version_warn': "0|1",
            u'client_lockdir': "path",
            u'user': "username",
            u'group': "groupname",
            u'umask': "umask",
            u'ratelimit': "Mb/s",
            u'network_timeout': "s",
            u'working_dir_recovery_method': "resume|use|delete",
            u'resume_partial': "0|1",
            u'client_can_delete': "0|1",
            u'client_can_force_backup': "0|1",
            u'client_can_list': "0|1",
            u'client_can_restore': "0|1",
            u'client_can_verify': "0|1",
            u'restore_client': "client",
            u'ssl_cert_ca': "path",
            u'ssl_cert': "path",
            u'ssl_key': "path",
            u'ssl_key_password': "password",
            u'ssl_cert_password': "password",
            u'ssl_ciphers': "cipher list",
            u'ssl_dhfile': "path",
            u'max_children': "number",
            u'max_status_children': "number",
            u'max_storage_subdirs': "number",
            u'timer_script': "path",
            u'timer_arg': "string",
            u'notify_success_script': "path",
            u'notify_success_arg': "string",
            u'notify_success_warnings_only': "0|1",
            u'notify_success_changes_only': "0|1",
            u'notify_failure_script': "path",
            u'notify_failure_arg': "string",
            u'dedup_group': "string",
            u'server_script_pre': "path",
            u'server_script_pre_arg': "string",
            u'server_script_pre_notify': "0|1",
            u'server_script_post': "path",
            u'server_script_post_arg': "string",
            u'server_script_post_notify': "0|1",
            u'server_script': "path",
            u'server_script_arg': "path",
            u'server_script_notify': "0|1",
            u'server_script_post_run_on_fail': "0|1",
            u'autoupgrade_dir': "path",
            u'ca_conf': "path",
            u'ca_name': "name",
            u'ca_server_name': "name",
            u'ca_burp_ca': "path",
        }
    values_server = {
            u'mode': ['client', 'server'],
            u'status_address': ['127.0.0.1', '::1'], #127.0.0.1 / ::1
            u'compression': ['gzip{0}'.format(x) for x in range(1, 10)],
            u'ssl_compression': ['zlib{0}'.format(x) for x in range(1, 10)],
            u'working_dir_recovery_method': ['use', 'delete', 'resume'],
        }
    files = [
            u'ssl_cert_ca',
            u'ssl_cert',
            u'ssl_key',
            u'ssl_dhfile',
            u'timer_script',
            u'notify_success_script',
            u'notify_failure_script',
            u'server_script_pre',
            u'server_script_post',
            u'server_script',
            u'ca_conf',
            u'ca_burp_ca',
        ]
    multi = [
            u'keep',
            u'restore_client',
            u'notify_success_arg',
            u'notify_failure_arg',
            u'timer_arg',
            u'server_script_arg',
            u'server_script_pre_arg',
            u'server_script_post_arg',
        ]
    boolean = [
            u'daemon',
            u'fork',
            u'directory_tree',
            u'password_check',
            u'syslog',
            u'stdout',
            u'hardlinked_archive',
            u'librsync',
            u'version_warn',
            u'path_length_warn',
            u'client_can_delete',
            u'client_can_diff',
            u'client_can_force_backup',
            u'client_can_list',
            u'client_can_restore',
            u'client_can_verify',
            u'notify_success_warnings_only',
            u'notify_success_changes_only',
            u'server_script_pre_notify',
            u'server_script_post_notify',
            u'server_script_notify',
            u'server_script_post_run_on_fail',
            u'monitor_browse_cache',
        ]
    integer = [
            u'port',
            u'status_port',
            u'max_hardlinks',
            u'protocol',
            u'ratelimit',
            u'network_timeout',
            u'max_children',
            u'max_status_children',
            u'max_storage_subdirs',
        ]
    string = [
            u'mode',
            u'address',
            u'status_address',
            u'directory',
            u'timestamp_format',
            u'pidfile',
            u'compression',
            u'client_lockdir',
            u'user',
            u'group',
            u'working_dir_recovery_method',
            u'ssl_key_password',
            u'ssl_ciphers',
            u'ssl_compression',
            u'dedup_group',
            u'ca_name',
            u'ca_server_name',
            u'ssl_cert_ca',
            u'ssl_cert',
            u'ssl_key',
            u'ssl_dhfile',
            u'ca_conf',
            u'ca_burp_ca',
            u'notify_success_script',
            u'notify_failure_script',
            u'server_script_pre',
            u'server_script_post',
            u'server_script',
            u'umask',
            u'hard_quota',
            u'soft_quota',
        ]
    server_doc = {
            u'ssl_compression': "Choose the level of zlib compression over SSL. Setting 0 or zlib0 turnsSSL compression off. Setting non-zero gives zlib5 compression (it is not currently possible for openssl to set any other level). The default is 5. 'gzip' is a synonym of 'zlib'.is a synonym of 'zlib'.",
            u'address': "Defines the main TCP address that the server listens on. The default is either '::' or '0.0.0.0', dependent upon compile time options.",
            u'status_address': "Defines the main TCP address that the server listens on for status requests. The default is either '::1' or '127.0.0.1', dependent upon compile time options.",
            u'mode': "Required to run in server mode.",
            u'port': "Defines the main TCP port that the server listens on.",
            u'status_port': "Defines the TCP port that the server listens on for status requests.",
            u'daemon': "Whether to daemonise. The default is 1.",
            u'fork': "Whether to fork children. The default is 1.",
            u'directory': "Path to the directory in which to store backups.",
            u'directory_tree': "When turned on (which is the default) and the client is on version 1.3.6 or greater, the structure of the storage directory will mimic that of the original filesystem on the client.",
            u'timestamp_format': "This allows you to tweak the format of the timestamps of individual backups. See 'man strftime' to see available substitutions. If this option is unset, burp uses \"%Y-%m-%d %H:%M:%S\".",
            u'password_check': "Allows you to turn client password checking on or off. The default is on. SSL certificates will still be checked if you turn passwords off. This option can be overridden by the client configuration files in clientconfdir on the server.",
            u'manual_delete': "If a path is given, the server will move directories to be deleted into the directory specified by the path, but will not actually delete them. The path must be on the same file system as the backup storage. The idea is that a busy server may be configured to run the deletions outside of the backup timebands, when the server is less busy, via a cron job. The default is unset, which means that the server will automatically delete the directories at the end of a backup. This option can be overridden by the client configuration files in clientconfdir on the server.",
            u'clientconfdir': "Path to the directory that contains client configuration files.",
            u'lockfile': "Path to the lockfile that ensures that two server processes cannot run simultaneously.",
            u'pidfile': "Synonym for lockfile.",
            u'syslog': "Log to syslog. Defaults to off.",
            u'stdout': "Log to stdout. Defaults to on.",
            u'keep': "Number of backups to keep. This can be overridden by the client configuration files in clientconfdir on the server. Specify multiple 'keep' entries on separate lines in order to keep multiple periods of backups. For example, assuming that you are doing a backup a day, keep=7 keep=4 keep=6 (on separate lines) will keep 7 daily backups, 4 weekly backups (7x4=28), and 6 multiples of 4 weeks (7x4x6=168) - roughly 6 monthly backups. Effectively, you will be guaranteed to be able to restore up to 168 days ago, with the number of available backups exponentially decreasing as you go back in time to that point. In this example, every 7th backup will be hardlinked to allow burp to safely delete intermediate backups when necessary. You can have as many 'keep' lines as you like, as long as they don't exceed 52560000 when multiplied together. That is, a backup every minute for 100 years.",
            u'hardlinked_archive': "On the server, defines whether to keep hardlinked files in the backups, or whether to generate reverse deltas and delete the original files. Can be set to either 0 (off) or 1 (on). Disadvantage: More disk space will be used Advantage: Restores will be faster, and since no reverse deltas need to be generated, the time and effort the server needs at the end of a backup is reduced.",
            u'max_hardlinks': "On the server, the number of times that a single file can be hardlinked. The bedup program also obeys this setting. The default is 10000.",
            u'librsync': "When set to 0, delta differencing will not take place. That is, when a file changes, the server will request the whole new file. The default is 1. This option can be overridden by the client configuration files in clientconfdir on the server.",
            u'compression': "Choose the level of gzip compression for files stored in backups. Setting 0 or gzip0 turns compression off. The default is gzip9. This option can be overridden by the client configuration files in clientconfdir on the server.",
            u'version_warn': "When this is on, which is the default, a warning will be issued when the client version does not match the server version. This option can be overridden by the client configuration files in clientconfdir on the server.",
            u'path_length_warn': "When this is on, which is the default, a warning will be issued when the client sends a path that is too long to replicate in the storage area tree structure. The file will still be saved in a numbered file outside of the tree structure, regardless of the setting of this option. This option can be overridden by the client configuration files in clientconfdir on the server.",
            u'client_lockdir': "Path to the directory in which to keep per-client lock files. By default, this is set to the path given by the 'directory' option.",
            u'user': "Run as a particular user. This can be overridden by the client configuration files in clientconfdir on the server.",
            u'group': "Run as a particular group. This can be overridden by the client configuration files in clientconfdir on the server.",
            u'umask': "Set the file creation umask. Default is 0022.",
            u'ratelimit': "Set the network send rate limit, in Mb/s. If this option is not given, burp will send data as fast as it can.",
            u'network_timeout': "Set the network timeout in seconds. If no data is sent or received over a period of this length, burp will give up. The default is 7200 seconds (2 hours).",
            u'working_dir_recovery_method': "This option tells the server what to do when it finds the working directory of an interrupted backup (perhaps somebody pulled the plug on the server, or something). This can be overridden by the client configurations files in clientconfdir on the server. Options are... <ul><li>delete: Just delete the old working directory.</li><li>use: Convert the working directory into a complete backup.</li><li>resume: Simply continue the previous backup from the point at which it left off, at file granularity. NOTE: If the client has changed its include/exclude configuration since the backup was interrupted, the recovery method will automatically switch to 'use'.</li></ul>",
            u'resume_partial': "Turn this on to enable 'resume partial' code. Requires 'working_dir_recovery_method=resume'. When resuming an interrupted transfer of a single file, it attempts to use previously transferred blocks of that file in order to be more efficient. However, situations have been reported where the file on the server side just gets bigger forever, so this feature now defaults to being turned off.",
            u'client_can_delete': "Turn this off to prevent clients from deleting backups with the '-a D' option. The default is that clients can delete backups. Restore clients can override this setting.",
            u'client_can_force_backup': "Turn this off to prevent clients from forcing backups with the '-a b' option. Timed backups will still work. The default is that clients can force backups.",
            u'client_can_list': "Turn this off to prevent clients from listing backups with the '-a l' option. The default is that clients can list backups. Restore clients can override this setting.",
            u'client_can_restore': "Turn this off to prevent clients from initiating restores with the '-a r' option. The default is that clients can initiate restores. Restore clients can override this setting.",
            u'client_can_verify': "Turn this off to prevent clients from initiating a verify job with the '-a v' option. The default is that clients can initiate a verify job. Restore clients can override this setting.",
            u'restore_client': "A client that is permitted to list, verify, restore and delete files belonging to any other client. You may specify multiple restore_clients. If this is too permissive, you may set a restore_client for individual original clients in the individual clientconfdir files. Note that restoring a backup from a Windows computer onto a Linux computer will currently leave the VSS headers in place at the beginning of each file. This will be addressed in a future version of burp.",
            u'ssl_cert_ca': "The path to the SSL CA certificate. This file will probably be the same on both the server and the client. The file should contain just the certificate in PEM format. For more information on this, and the other ssl_* options, please see docs/burp_ca.txt.",
            u'ssl_cert': "The path to the server SSL certificate. It works for me when the file contains the concatenation of the certificate and private key in PEM format.",
            u'ssl_key': "The path to the server SSL private key in PEM format.",
            u'ssl_key_password': "The SSL key password.",
            u'ssl_cert_password': "Synonym for ssl_key_password.",
            u'ssl_ciphers': "Allowed SSL ciphers. See openssl ciphers for details.",
            u'ssl_dhfile': "Path to Diffie-Hellman parameter file. To generate one with openssl, use a command like this: openssl dhparam -out dhfile.pem -5 1024",
            u'max_children': "Defines the number of child processes to fork (the number of clients that can simultaneously connect. The default is 5.",
            u'max_status_children': "Defines the number of status child processes to fork (the number of status clients that can simultaneously connect. The default is 5.",
            u'max_storage_subdirs': "Defines the number of subdirectories in the data storage areas. The maximum number of subdirectories that ext3 allows is 32000. If you do not set this option, it defaults to 30000.",
            u'timer_script': "Path to the script to run when a client connects with the timed backup option. If the script exits with code 0, a backup will run. The first two arguments are the client name and the path to the 'current' storage directory. The next three arguments are reserved, and user arguments are appended after that. An example timer script is provided. The timer_script option can be overridden by the client configuration files in clientconfdir on the server.",
            u'timer_arg': "A user-definable argument to the timer script. You can have many of these. The timer_arg options can be overridden by the client configuration files in clientconfdir on the server.",
            u'notify_success_script': "Path to the script to run when a backup succeeds. User arguments are appended after the first five reserved arguments. An example notify script is provided. The notify_success_script option can be overriddden by the client configuration files in clientconfdir on the server.",
            u'notify_success_arg': "A user-definable argument to the notify success script. You can have many of these. The notify_success_arg options can be overriddden by the client configuration files in clientconfdir on the server.",
            u'notify_success_warnings_only': "Set to 1 to send success notifications when there were warnings. If this and notify_success_changes_only are not turned on, success notifications are always sent.",
            u'notify_failure_script': "The same as notify_success_script, but for backups that failed.",
            u'notify_failure_arg': "The same as notify_success_arg, but for backups that failed.",
            u'dedup_group': "Enables you to group clients together for file deduplication purposes. For example, you might want to set 'dedup_group=xp' for each Windows XP client, and then run the bedup program on a cron job every other day with the option '-g xp'.",
            u'server_script_pre': "Path to a script to run on the server after each successfully authenticated connection but before any work is carried out. The arguments to it are 'pre', '(client command)', 'reserved3' to 'reserved5', and then arguments defined by server_script_pre_arg. If the script returns non-zero, the task asked for by the client will not be run. This command and related options can be overriddden by the client configuration files in clientconfdir on the server.",
            u'server_script_pre_arg': "A user-definable argument to the server pre script. You can have many of these.",
            u'server_script_pre_notify': "Turn on to send a notification email when the server pre script returns non-zero. The output of the script will be included in the email. The default is off. Most people will not want this turned on because clients usually contact the server at 20 minute intervals and this could cause a lot of emails to be generated. Requires the notify_failure options to be set.",
            u'server_script_post': "Path to a script to run on the server before the client disconnects. The arguments to it are 'post', '(client command)', 'reserved3' to 'reserved5', and then arguments defined by server_script_post_arg. This command and related options can be overriddden by the client configuration files in clientconfdir on the server.",
            u'server_script_post_arg': "A user-definable argument to the server post script. You can have many of these.",
            u'server_script_post_notify': "Turn on to send a notification email when the server post script returns non-zero. The output of the script will be included in the email. The default is off. Requires the notify_failure options to be set.",
            u'server_script': "You can use this to save space in your config file when you want to run the same server script twice. It overrides server_script_pre and server_script_post. This command and related options can be overriddden by the client configuration files in clientconfdir on the server.",
            u'server_script_arg': "Goes with server_script and overrides server_script_pre_arg and server_script_post_arg.",
            u'server_script_notify': "Turn on to send a notification emails when the server pre and post scripts return non-zero. The output of the script will be included it the email. The default is off. Requires the notify_failure options to be set.",
            u'server_script_post_run_on_fail': "If this is set to 1, server_script_post will always be run. The default is 0, which means that if the task asked for by the client fails, server_script_post will not be run.",
            u'autoupgrade_dir': "Path to autoupgrade directory from which upgrades are downloaded. The option can be left unset in order not to autoupgrade clients. Please see docs/autoupgrade.txt in the source package for more help with this option.",
            u'ca_conf': "Path to certificate authority configuration file. The CA configuration file will usually be /etc/burp/CA.cnf. The CA directory indicated by CA.cnf will usually be /etc/burp/CA. If ca_conf is set and the CA directory does not exist, the server will create, populate it, and the paths indicated by ssl_cert_ca, ssl_cert, ssl_key and ssl_dhfile will be overwritten. For more detailed information on this and the other ca_* options, please see docs/burp_ca.txt.",
            u'ca_name': "Name of the CA that the server will generate when using the ca_conf option.",
            u'ca_server_name': "The name that the server will put into its own SSL certficates when using the ca_conf option.",
            u'ca_burp_ca': "Path to the burp_ca script when using the ca_conf option.",
            u'soft_quota': "A warning will be issued when the estimated size of all files is greater than the specified size and smaller than hard_quota. Example: 'soft_quota = 95Gb'. Set to 0 (the default) to have no warning.",
            u'hard_quota': "Do not back up the client if the estimated size of all files is greater than the specified size. Example: 'hard_quota = 100Gb'. Set to 0 (the default) to have no limit.",
        }

    #def __init__(self, app=None, conf=None):
        #self._logger('info', 'temporary dir: %s', self.tmpdir)


    def _readfile(self, f=None, sourced=False):
        if not f:
            return []
        if (f != self.conf or sourced) and not f.startswith('/'):
            root = os.path.dirname(self.conf)
            f = os.path.join(root, f)
        self._logger('debug', 'reading file: %s', f)
        with codecs.open(f, 'r', 'utf-8') as ff:
            ret = [x.strip('\n') for x in ff.readlines()]

        return ret

    def _parse_lines_srv(self, fi):
        other_files = []
        dic = []
        boolean = []
        multi = []
        integer = []
        for l in fi:
            if re.match('^\s*#', l):
                continue
            r = re.search('\s*(\S+)\s*=?\s*(.*)$', l)
            if r:
                key = r.group(1)
                val = r.group(2)
                if key in self.boolean:
                    boolean.append({'name': key, 'value': bool(val)})
                    continue
                elif key in self.integer:
                    integer.append({'name': key, 'value': int(val)})
                    continue
                if key == u'.':
                    other_files.append(val)
                    continue
                if key in self.multi:
                    found = False
                    for m in multi:
                        if m['name'] == key:
                            m['value'].append(val)
                            found = True
                            break
                    if not found:
                        multi.append({'name': key, 'value': [val]})
                    continue
                dic.append({'name': key, 'value': val})

        return dic, boolean, multi, integer, other_files

    def read_server_conf(self):
        if not self.conf:
            return []
        self.content = []
        res = {}
        other_files = []
        f = self._readfile(self.conf)

        tmp, boolean, multi, integer, other_files = self._parse_lines_srv(f)
        res['common'] = tmp
        res['boolean'] = boolean
        res['integer'] = integer
        res['multi'] = multi

        if other_files:
            while True:
                other_files2 = []
                for fi in other_files:
                    f = self._readfile(fi)
                    tmp, boolean, multi, integer, dummy2 = self._parse_lines_srv(f)
                    res['common'].update(tmp)
                    res['boolean'] += boolean
                    res['multi'].update(multi)
                    res['integer'] += integer
                    other_files2 += dummy2
                if other_files2:
                    other_files = other_files2
                else:
                    break

        return res

    def store_server_conf(self, data):
        if not self.conf:
            return [[0, 'Sorry, no configuration file defined']]
        ref = '{}.bui.init.back'.format(self.conf)
        if not os.path.isfile(ref):
            try:
                shutil.copy(self.conf, ref)
            except Exception, e:
                return [[2, str(e)]]

        with codecs.open(self.conf, 'w', 'utf-8') as f:
            f.write('# Auto-generated configuration using Burp-UI\n')
            for key in data.keys():
                if key in self.boolean:
                    val = 0
                    if data.get(key):
                        val = 1
                    f.write('{} = {}\n'.format(key, val))
                elif key in self.multi:
                    for val in data.getlist(key):
                        f.write('{} = {}\n'.format(key, val))
                else:
                    f.write('{} = {}\n'.format(key, data.get(key)))

        return [[0, 'Configuration successfully saved.']]

    def get_priv_attr(self, key):
        try:
            return getattr(self, key)
        except:
            return None



# -*- coding: utf8 -*-
import os
import struct
import select
import json
import time
import ConfigParser
import SocketServer
from threading import Thread

g_port = '10000'
g_bind = '::'
g_ssl  = 'False'
g_version = '1'
g_sslcert = ''
g_sslkey  = ''
g_password = 'password'

class BUIAgent:
    def __init__(self, conf=None, debug=False):
        global g_port, g_bind, g_ssl, g_version, g_sslcert, g_sslkey, g_password
        self.conf = conf
        self.dbg = debug
        print 'conf: '+self.conf
        print 'debug: '+str(self.dbg)
        if not conf:
            raise IOError('No configuration file found')

        config = ConfigParser.ConfigParser({'port': g_port,'bind': g_bind,
                    'ssl': g_ssl, 'sslcert': g_sslcert, 'sslkey': g_sslkey,
                    'version': g_version, 'password': g_password})
        with open(conf) as fp:
            config.readfp(fp)
            try:
                self.port = config.getint('Global', 'port')
                self.bind = config.get('Global', 'bind')
                self.vers = config.getint('Global', 'version')
                try:
                    self.ssl = config.getboolean('Global', 'ssl')
                except ValueError:
                    self.app.logger.error("Wrong value for 'ssl' key! Assuming 'false'")
                    self.ssl = False
                self.sslcert = config.get('Global', 'sslcert')
                self.sslkey = config.get('Global', 'sslkey')
                self.password = config.get('Global', 'password')
            except ConfigParser.NoOptionError, e:
                raise e

        module = 'burpui.misc.backend.burp{0}'.format(self.vers)
        try:
            mod = __import__(module, fromlist=['Burp'])
            Client = mod.Burp
            self.backend = Client(conf=conf)
        except Exception, e:
            self.app.logger.error('Failed loading backend for Burp version %d: %s', self.vers, str(e))
            sys.exit(2)

        self.methods = {
                'status': self.backend.status,
                'get_backup_logs': self.backend.get_backup_logs,
                'get_counters': self.backend.get_counters,
                'is_backup_running': self.backend.is_backup_running,
                'is_one_backup_running': self.backend.is_one_backup_running,
                'get_all_clients': self.backend.get_all_clients,
                'get_client': self.backend.get_client,
                'get_tree': self.backend.get_tree,
                'restore_files': self.backend.restore_files,
                'read_conf': self.backend.read_conf,
                'store_conf': self.backend.store_conf,
                'get_parser_attr': self.backend.get_parser_attr
            }

        self.server = AgentServer((self.bind, self.port), AgentTCPHandler, self)

    def run(self):
        try:
            self.server.serve_forever()
        except KeyboardInterrupt:
            sys.exit(0)

    def debug(self, msg, *args):
        if self.dbg:
            print msg % (args)

class AgentTCPHandler(SocketServer.BaseRequestHandler):
    "One instance per connection.  Override handle(self) to customize action."
    def handle(self):
        # self.request is the client connection
        self.server.agent.debug('===============>')
        try:
            r, _, _ = select.select([self.request], [], [], 5)
            if not r:
                raise Exception ('Socket timed-out')
            lengthbuf = self.request.recv(8)
            length, = struct.unpack('!Q', lengthbuf)
            data = self.recvall(length)
            self.server.agent.debug('####################')
            self.server.agent.debug('recv: %s', data)
            self.server.agent.debug('####################')
            j = json.loads(data)
            _, w, _ = select.select([], [self.request], [], 5)
            if not w:
                raise Exception ('Socket timed-out')
            if j['password'] != self.server.agent.password:
                self.server.agent.debug('-----> Wrong Password <-----')
                self.request.sendall('KO')
                return
            if j['func'] not in self.server.agent.methods:
                self.server.agent.debug('-----> Wrong method <-----')
                self.request.sendall('KO')
                return
            self.request.sendall('OK')
            if j['func'] == 'restore_files':
                res = self.server.agent.methods[j['func']](**j['args'])
            else:
                if j['args']:
                    res = json.dumps(self.server.agent.methods[j['func']](**j['args']))
                else:
                    res = json.dumps(self.server.agent.methods[j['func']]())
            self.server.agent.debug('####################')
            self.server.agent.debug('result: %s', res)
            self.server.agent.debug('####################')
            _, w, _ = select.select([], [self.request], [], 5)
            if not w:
                raise Exception ('Socket timed-out')
            if j['func'] == 'restore_files':
                size = os.path.getsize(res)
                self.request.sendall(struct.pack('!Q', size))
                with open(res, 'rb') as f:
                    buf = f.read(1024)
                    while buf:
                        self.server.agent.debug('sending %d Bytes', len(buf))
                        self.request.sendall(buf)
                        buf = f.read(1024)
                        _, w, _ = select.select([], [self.request], [], 5)
                        if not w:
                            raise Exception ('Socket timed-out')
            else:
                self.request.sendall(struct.pack('!Q', len(res)))
                self.request.sendall(res)
            self.request.close()
        except Exception, e:
            self.server.agent.debug('ERROR: %s', str(e))
        finally:
            self.server.agent.debug('<===============')

    def recvall(self, length=1024):
        buf = b''
        bsize = 1024
        received = 0
        if length < bsize:
            bsize = length
        while received < length:
            newbuf = self.request.recv(bsize)
            if not newbuf:
                time.sleep(0.1)
                continue
            buf += newbuf
            received += len(newbuf)
        return buf

class AgentServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
    # Ctrl-C will cleanly kill all spawned threads
    daemon_threads = True
    # much faster rebinding
    allow_reuse_address = True

    def __init__(self, server_address, RequestHandlerClass, agent=None):
        self.agent = agent
        SocketServer.TCPServer.__init__(self, server_address, RequestHandlerClass)

    def get_request(self):
        if self.agent.ssl: 
            import ssl
            (newsocket, fromaddr) = SocketServer.TCPServer.get_request(self)
            connstream = ssl.wrap_socket(newsocket,
                                        server_side=True,
                                        certfile=self.agent.sslcert,
                                        keyfile=self.agent.sslkey,
                                        ssl_version=ssl.PROTOCOL_SSLv23)
            return connstream, fromaddr
        # if we don't use ssl, use the 'super' method
        return SocketServer.TCPServer.get_request(self)




# -*- coding: utf8 -*-
import re
import copy
import socket
import select
import sys
import json
import time
import struct
import ConfigParser

from burpui.misc.backend.interface import BUIbackend, BUIserverException

class Burp(BUIbackend):

    def __init__(self, app=None, conf=None):
        self.app = app
        self.servers = {}
        self.app.config['SERVERS'] = []
        self.running = {}
        if conf:
            config = ConfigParser.ConfigParser()
            with open(conf) as fp:
                config.readfp(fp)
                for sec in config.sections():
                    r = re.match('^Agent:(.+)$', sec)
                    if r:
                        try:
                            host = config.get(sec, 'host')
                            port = config.getint(sec, 'port')
                            password = config.get(sec, 'password')
                            ssl = config.getboolean(sec, 'ssl')
                        except Exception, e:
                            self.app.logger.error(str(e))
                        self.servers[r.group(1)] = NClient(app, host, port, password, ssl)

        self.app.logger.debug(self.servers)
        for key, serv in self.servers.iteritems():
            self.app.config['SERVERS'].append(key)

    """
    Utilities functions
    """

    def status(self, query='\n', agent=None):
        """
        status connects to the burp status port, ask the given 'question' and
        parses the output in an array
        """
        return self.servers[agent].status(query)

    def get_backup_logs(self, n, c, forward=False, agent=None):
        """
        parse_backup_log parses the log.gz of a given backup and returns a dict
        containing different stats used to render the charts in the reporting view
        """
        return self.servers[agent].get_backup_logs(n, c, forward)

    def get_counters(self, name=None, agent=None):
        """
        get_counters parses the stats of the live status for a given client and
        returns a dict
        """
        return self.servers[agent].get_counters(name)

    def is_backup_running(self, name=None, agent=None):
        """
        is_backup_running returns True if the given client is currently running a
        backup
        """
        return self.servers[agent].is_backup_running(name)

    def is_one_backup_running(self, agent=None):
        """
        is_one_backup_running returns a list of clients name that are currently
        running a backup
        """
        r = []
        if agent:
            r = self.servers[agent].is_one_backup_running(agent)
            self.running[agent] = r
        else:
            r = {}
            for a in self.servers:
                r[a] = self.servers[a].is_one_backup_running(a)
            self.running = r
        return r

    def get_all_clients(self, agent=None):
        """
        get_all_clients returns a list of dict representing each clients with their
        name, state and last backup date
        """
        if agent not in self.servers:
            return []
        return self.servers[agent].get_all_clients()

    def get_client(self, name=None, agent=None):
        """
        get_client returns a list of dict representing the backups (with its number
        and date) of a given client
        """
        return self.servers[agent].get_client(name)

    def get_tree(self, name=None, backup=None, root=None, agent=None):
        """
        get_tree returns a list of dict representing files/dir (with their attr)
        within a given path
        """
        return self.servers[agent].get_tree(name, backup, root)

    def restore_files(self, name=None, backup=None, files=None, strip=None, agent=None):
        return self.servers[agent].restore_files(name, backup, files, strip)

    def read_conf(self, agent=None):
        return self.servers[agent].read_conf()

    def store_conf(self, data, agent=None):
        return self.servers[agent].store_conf(data)

    def get_parser_attr(self, attr=None, agent=None):
        return self.servers[agent].get_parser_attr(attr)

class NClient(BUIbackend):

    def __init__(self, app=None, host=None, port=None, password=None, ssl=None):
        self.host = host
        self.port = port
        self.password = password
        self.ssl = ssl
        self.connected = False
        self.app = app

    def conn(self):
        try:
            if self.connected:
                return
            self.sock = self.do_conn()
            self.connected = True
            self.app.logger.debug('OK, connected to agent %s:%s', self.host, self.port)
        except Exception, e:
            self.connected = False
            self.app.logger.error('Could not connect to %s:%s => %s', self.host, self.port, str(e))

    def do_conn(self):
        ret = None
        if self.ssl:
            import ssl
            s = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
            ret = ssl.wrap_socket(s, cert_reqs=ssl.CERT_NONE, ssl_version=ssl.PROTOCOL_SSLv23)
            try:
                ret.connect((self.host, self.port))
            except Exception, e:
                self.app.logger.error('ERROR: %s', str(e))
                raise e
        else:
            ret = socket.create_connection((self.host, self.port))
        return ret

    def ping(self):
        self.conn()
        res = self.connected
        return res

    def close(self, force=True):
        if self.connected and force:
            self.sock.close()
        self.connected = False

    def do_command(self, data=None):
        self.conn()
        res = '[]'
        toclose = True
        if not data:
            return res
        try:
            data['password'] = self.password
            raw = json.dumps(data)
            length = len(raw)
            self.sock.sendall(struct.pack('!Q', length))
            self.sock.sendall(raw)
            self.app.logger.debug("Sending: %s", raw)
            r, _, _ = select.select([self.sock], [], [], 5)
            if not r:
                raise Exception ('Socket timed-out')
            tmp = self.sock.recv(2)
            self.app.logger.debug("recv: '%s'", tmp)
            if 'OK' != tmp:
                self.app.logger.debug('Ooops, unsuccessful!')
                return res
            self.app.logger.debug("Data sent successfully")
            r, _, _ = select.select([self.sock], [], [], 5)
            if not r:
                raise Exception ('Socket timed-out')
            lengthbuf = self.sock.recv(8)
            length, = struct.unpack('!Q', lengthbuf)
            if data['func'] == 'restore_files':
                toclose = False
                res = (self.sock, length)
            else:
                r, _, _ = select.select([self.sock], [], [], 5)
                if not r:
                    raise Exception ('Socket timed-out')
                res = self.sock.recv(length)
        except Exception, e:
            self.app.logger.error(str(e))
        finally:
            self.close(toclose)
            return res

    """
    Utilities functions
    """

    def status(self, query='\n', agent=None):
        """
        status connects to the burp status port, ask the given 'question' and
        parses the output in an array
        """
        data = {'func': 'status', 'args': {'query': query}}
        return json.loads(self.do_command(data))

    def get_backup_logs(self, n, c, forward=False, agent=None):
        """
        parse_backup_log parses the log.gz of a given backup and returns a dict
        containing different stats used to render the charts in the reporting view
        """
        data = {'func': 'get_backup_logs', 'args': {'n': n, 'c': c, 'forward': forward}}
        return json.loads(self.do_command(data))

    def get_counters(self, name=None, agent=None):
        """
        get_counters parses the stats of the live status for a given client and
        returns a dict
        """
        data = {'func': 'get_counters', 'args': {'name': name}}
        return json.loads(self.do_command(data))

    def is_backup_running(self, name=None, agent=None):
        """
        is_backup_running returns True if the given client is currently running a
        backup
        """
        data = {'func': 'is_backup_running', 'args': {'name': name}}
        return json.loads(self.do_command(data))

    def is_one_backup_running(self, agent=None):
        """
        is_one_backup_running returns a list of clients name that are currently
        running a backup
        """
        data = {'func': 'is_one_backup_running', 'args': {'agent': agent}}
        return json.loads(self.do_command(data))

    def get_all_clients(self, agent=None):
        """
        get_all_clients returns a list of dict representing each clients with their
        name, state and last backup date
        """
        data = {'func': 'get_all_clients', 'args': None}
        return json.loads(self.do_command(data))

    def get_client(self, name=None, agent=None):
        """
        get_client returns a list of dict representing the backups (with its number
        and date) of a given client
        """
        data = {'func': 'get_client', 'args': {'name': name}}
        return json.loads(self.do_command(data))

    def get_tree(self, name=None, backup=None, root=None, agent=None):
        """
        get_tree returns a list of dict representing files/dir (with their attr)
        within a given path
        """
        data = {'func': 'get_tree', 'args': {'name': name, 'backup': backup, 'root': root}}
        return json.loads(self.do_command(data))

    def restore_files(self, name=None, backup=None, files=None, strip=None, agent=None):
        data = {'func': 'restore_files', 'args': {'name': name, 'backup': backup, 'files': files, 'strip': strip}}
        return self.do_command(data)

    def read_conf(self, agent=None):
        data = {'func': 'read_conf', 'args': None}
        return json.loads(self.do_command(data))

    def store_conf(self, data, agent=None):
        data = {'func': 'store_conf', 'args': {'data': data}}
        return json.loads(self.do_comman(data))

    def get_parser_attr(self, attr=None, agent=None):
        data = {'func': 'get_parser_attr', 'args': {'attr': attr}}
        return json.loads(self.do_command(data))



#!/usr/bin/env python
# -*- coding: utf8 -*-
import sys
import os
import unittest
import urllib2
import pprint
from flask.ext.testing import LiveServerTestCase, TestCase

sys.path.append('{0}/..'.format(os.path.join(os.path.dirname(os.path.realpath(__file__)))))

from burpui import app, bui, login_manager, init as BUIinit

class BurpuiLiveTestCase(LiveServerTestCase):

	def create_app(self):
		conf = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../burpui.cfg')
		app.config['TESTING'] = True
		app.config['LOGIN_DISABLED'] = True
		app.config['LIVESERVER_PORT'] = 5001
		app.config['CFG'] = conf
		bui.setup(conf)
		login_manager.init_app(app)
		return app

	def setUp(self):
		print '\nBegin Test 1\n'

	def tearDown(self):
		print '\nTest 1 Finished!\n'

	def test_server_is_up_and_running(self):
		response = urllib2.urlopen(self.get_server_url())
		self.assertEqual(response.code, 200)

class BurpuiAPITestCase(TestCase):

	def setUp(self):
		print '\nBegin Test 2\n'
	
	def tearDown(self):
		print '\nTest 2 Finished!\n'
	
	def create_app(self):
		conf = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'test2.cfg')
		app.config['TESTING'] = True
		app.config['LOGIN_DISABLED'] = True
		app.config['CFG'] = conf
		bui.setup(conf)
		login_manager.init_app(app)
		return app

	def test_no_clients(self):
		response = self.client.get('/api/clients.json')
		self.assertEquals(response.json, {u'notif':[[2, u'Cannot contact burp server at 127.0.0.1:9999']]})
	
	def test_config_parsing(self):
		response = self.client.get('/api/server-config')
		self.assertEquals(response.json, dict(results=[],
												 boolean=bui.cli.get_parser_attr('boolean'),
												 string=bui.cli.get_parser_attr('string'),
												 integer=bui.cli.get_parser_attr('integer'),
												 multi=bui.cli.get_parser_attr('multi'),
												 server_doc=bui.cli.get_parser_attr('server_doc'),
												 suggest=bui.cli.get_parser_attr('values_server'),
												 placeholders=bui.cli.get_parser_attr('placeholders'),
												 defaults=bui.cli.get_parser_attr('defaults_server')))
	
	def test_restore(self):
		response = self.client.post('/api/restore/dummy/1', data=dict(strip=False))
		self.assert500(response)
	
	def test_running_clients(self):
		response = self.client.get('/api/running-clients.json')
		self.assertEquals(response.json, dict(results=[]))
	
	def test_live_rendering(self):
		response = self.client.get('/api/render-live-template/toto')
		self.assert404(response)
		response = self.client.get('/api/render-live-template')
		self.assert500(response)

	def test_servers_json(self):
		response = self.client.get('/api/servers.json')
		self.assertEquals(response.json, dict(results=[]))

	def test_live(self):
		response = self.client.get('/api/live.json')
		self.assertEquals(response.json, dict(results=[]))
	
	def test_running(self):
		response = self.client.get('/api/running.json')
		self.assertEquals(response.json, dict(results=False))
	
	def test_client_tree(self):
		response = self.client.get('/api/client-tree.json/toto/1')
		self.assertEquals(response.json, {u'notif':[[2, u'Cannot contact burp server at 127.0.0.1:9999']]})
	
	def test_clients_report_json(self):
		response = self.client.get('/api/clients-report.json')
		self.assertEquals(response.json, {u'notif':[[2, u'Cannot contact burp server at 127.0.0.1:9999']]})
	
	def test_client_stat_json(self):
		response = self.client.get('/api/client-stat.json/toto')
		self.assertEquals(response.json, {u'notif':[[2, u'Cannot contact burp server at 127.0.0.1:9999']]})
		response = self.client.get('/api/client-stat.json/toto/1')
		self.assertEquals(response.json, {u'notif':[[2, u'Cannot contact burp server at 127.0.0.1:9999']]})
	
	def test_client_json(self):
		response = self.client.get('/api/client.json/toto')
		self.assertEquals(response.json, {u'notif':[[2, u'Cannot contact burp server at 127.0.0.1:9999']]})

class BurpuiRoutesTestCase(TestCase):

	def setUp(self):
		print '\nBegin Test 3\n'

	def tearDown(self):
		print '\nTest 3 Finished!\n'

	def create_app(self):
		conf = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../burpui.cfg')
		app.config['TESTING'] = True
		app.config['LOGIN_DISABLED'] = True
		app.config['LIVESERVER_PORT'] = 5001
		app.config['CFG'] = conf
		bui.setup(conf)
		bui.cli.port = 9999
		login_manager.init_app(app)
		return app
	
	def test_config_render(self):
		response = self.client.get('/settings')
		assert 'Burp Configuration' in response.data
	
	def test_live_monitor(self):
		response = self.client.get('/live-monitor', follow_redirects=True)
		assert 'Sorry, there are no running backups' in response.data

class BurpuiLoginTestCase(TestCase):

	def setUp(self):
		print '\nBegin Test 4\n'

	def tearDown(self):
		print '\nTest 4 Finished!\n'

	def login(self, username, password):
		return self.client.post('/login', data=dict(
			username=username,
			password=password
		), follow_redirects=True)

	def create_app(self):
		conf = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../burpui.cfg')
		BUIinit(conf, False, False)
		app.config['TESTING'] = True
		app.config['LIVESERVER_PORT'] = 5001
		app.config['WTF_CSRF_ENABLED'] = False
		bui.cli.port = 9999
		login_manager.init_app(app)
		return app

	def test_login_ok(self):
		rv = self.login('admin', 'admin')
		assert 'Logged in successfully' in rv.data
	
	def test_login_ko(self):
		rv = self.login('admin', 'toto')
		assert 'Wrong username or password' in rv.data

	def test_login_no_user(self):
		rv = self.login('toto', 'toto')
		assert 'Wrong username or password' in rv.data

if __name__ == '__main__':
	unittest.main()



# -*- coding: utf8 -*-

class BUIparser:
    def __init__(self, app=None, conf=None):
        self.app = app
        self.conf = conf

    def read_server_conf(self):
        raise NotImplementedError("Sorry, the current Parser does not implement this method!")

    def store_server_conf(self, data):
        raise NotImplementedError("Sorry, the current Parser does not implement this method!")

    def get_priv_attr(self, key):
        raise NotImplementedError("Sorry, the current Parser does not implement this method!")



# -*- coding: utf8 -*-
import re
import os
import socket
import time
import json
import datetime
import ConfigParser
import shutil
import subprocess
import zipfile
import codecs

from burpui.misc.utils import human_readable as _hr, BUIlogging
from burpui.misc.backend.interface import BUIbackend, BUIserverException
from burpui.misc.parser.burp1 import Parser

g_burpport    = '4972'
g_burphost    = '::1'
g_tmpdir      = u'/tmp/buirestore'
g_burpbin     = u'/usr/sbin/burp'
g_stripbin    = u'/usr/sbin/vss_strip'
g_burpconfcli = None
g_burpconfsrv = u'/etc/burp/burp-server.conf'

class Burp(BUIbackend, BUIlogging):
    states = {
        'i': 'idle',
        'r': 'running',
        'c': 'client crashed',
        'C': 'server crashed',
        '1': 'scanning',
        '2': 'backup',
        '3': 'merging',
        '4': 'shuffling',
        '7': 'listing',
        '8': 'restoring',
        '9': 'verifying',
        '0': 'deleting'
        }

    counters = [
        'phase',
        'Total',
        'Files',
        'Files (encrypted)',
        'Metadata',
        'Metadata (enc)',
        'Directories',
        'Softlink',
        'Hardlink',
        'Special files',
        'VSS header',
        'VSS header (enc)',
        'VSS footer',
        'VSS footer (enc)',
        'Grand total',
        'warning',
        'estimated_bytes',
        'bytes',
        'bytes_in',
        'bytes_out',
        'start',
        'path'
       ]

    def __init__(self, app=None, conf=None):
        global g_burpport, g_burphost, g_tmpdir, g_burpbin, g_stripbin, g_burpconfcli, g_burpconfsrv
        self.app = app
        self.host = g_burphost
        self.port = int(g_burpport)
        self.burpbin = g_burpbin
        self.stripbin = g_stripbin
        self.tmpdir = g_tmpdir
        self.burpconfcli = g_burpconfcli
        self.burpconfsrv = g_burpconfsrv
        self.running = []
        if conf:
            config = ConfigParser.ConfigParser({'bport': g_burpport, 'bhost': g_burphost, 'tmpdir': g_tmpdir, 'burpbin': g_burpbin, 'stripbin': g_stripbin, 'bconfcli': g_burpconfcli, 'bconfsrv': g_burpconfsrv})
            with codecs.open(conf, 'r', 'utf-8') as fp:
                config.readfp(fp)
                try:
                    self.port = config.getint('Burp1', 'bport')
                    self.host = config.get('Burp1', 'bhost')
                    tdir = config.get('Burp1', 'tmpdir')
                    bbin = config.get('Burp1', 'burpbin')
                    strip = config.get('Burp1', 'stripbin')
                    confcli = config.get('Burp1', 'bconfcli')
                    confsrv = config.get('Burp1', 'bconfsrv')

                    if confcli and not os.path.isfile(confcli):
                        self._logger('warning', "The file '%s' does not exist", confcli)
                        confcli = None

                    if confsrv and not os.path.isfile(confsrv):
                        self._logger('warning', "The file '%s' does not exist", confsrv)
                        confsrv = None

                    if self.host not in ['127.0.0.1', '::1']:
                        self._logger('warning', "Invalid value for 'bhost'. Must be '127.0.0.1' or '::1'. Falling back to '%s'", g_burphost)
                        self.host = g_burphost

                    if not strip.startswith('/'):
                        self._logger('warning', "Please provide an absolute path for the 'stripbin' option. Fallback to '%s'", g_stripbin)
                        strip = g_stripbin
                    elif not re.match('^\S+$', strip):
                        self._logger('warning', "Incorrect value for the 'stripbin' option. Fallback to '%s'", g_stripbin)
                        strip = g_stripbin
                    elif not os.path.isfile(strip) or not os.access(strip, os.X_OK):
                        self._logger('warning', "'%s' does not exist or is not executable. Fallback to '%s'", strip, g_stripbin)
                        strip = g_stripbin

                    if not os.path.isfile(strip) or not os.access(strip, os.X_OK):
                        self._logger('error', "Ooops, '%s' not found or is not executable", strip)
                        strip = None
                        
                    if not bbin.startswith('/'):
                        self._logger('warning', "Please provide an absolute path for the 'burpbin' option. Fallback to '%s'", g_burpbin)
                        bbin = g_burpbin
                    elif not re.match('^\S+$', bbin):
                        self._logger('warning', "Incorrect value for the 'burpbin' option. Fallback to '%s'", g_burpbin)
                        bbin = g_burpbin
                    elif not os.path.isfile(bbin) or not os.access(bbin, os.X_OK):
                        self._logger('warning', "'%s' does not exist or is not executable. Fallback to '%s'", bbin, g_burpbin)
                        bbin = g_burpbin

                    if not os.path.isfile(bbin) or not os.access(bbin, os.X_OK):
                        self._logger('error', "Ooops, '%s' not found or is not executable", bbin)
                        bbin = None

                    if not tdir.startswith('/'):
                        self._logger('warning', "Please provide an absolute path for the 'tmpdir' option. Fallback to '%s'", g_tmpdir)
                        tdir = g_tmpdir
                    elif not re.match('^\S+$', tdir):
                        self._logger('warning', "Incorrect value for the 'tmpdir' option. Fallback to '%s'", g_tmpdir)
                        tdir = g_tmpdir
                    elif os.path.isdir(tdir) and os.listdir(tdir) and not self.app.config.get('TESTING'):
                        raise Exception("'{0}' is not empty!".format(tdir))
                    elif os.path.isdir(tdir) and not os.access(tdir, os.W_OK|os.X_OK):
                        self._logger('warning', "'%s' is not writable. Fallback to '%s'", tdir, g_tmpdir)
                        tdir = g_tmpdir

                    self.burpbin = bbin
                    self.tmpdir = tdir
                    self.stripbin = strip
                    self.burpconfcli = confcli
                    self.burpconfsrv = confsrv
                except ConfigParser.NoOptionError, e:
                    self._logger('error', str(e))
                except ConfigParser.NoSectionError, e:
                    self._logger('error', str(e))

        self.parser = Parser(self.app, self.burpconfsrv)

        self.family = self._get_inet_family(self.host)
        self._test_burp_settings(self.host)

        self._logger('info', 'burp port: %d', self.port)
        self._logger('info', 'burp host: %s', self.host)
        self._logger('info', 'burp binary: %s', self.burpbin)
        self._logger('info', 'strip binary: %s', self.stripbin)
        self._logger('info', 'temporary dir: %s', self.tmpdir)
        self._logger('info', 'burp conf cli: %s', self.burpconfcli)
        self._logger('info', 'burp conf srv: %s', self.burpconfsrv)

    """
    Utilities functions
    """

    def _get_inet_family(self, addr):
        if addr == '127.0.0.1':
            return socket.AF_INET
        else:
            return socket.AF_INET6

    def _test_burp_settings(self, addr, retry=False):
        family = self._get_inet_family(addr)
        try:
            s = socket.socket(family, socket.SOCK_STREAM)
            s.connect((addr, self.port))
            s.close()
            return True
        except socket.error:
            self._logger('warning', 'Cannot contact burp server at %s:%s', addr, self.port)
            if not retry:
                new_addr = ''
                if self.host == '127.0.0.1':
                    new_addr = '::1'
                else:
                    new_addr = '127.0.0.1'
                self._logger('info', 'Trying %s:%s instead', new_addr, self.port)
                if self._test_burp_settings(new_addr, True):
                    self._logger('info', '%s:%s is reachable, switching to it for this runtime', new_addr, self.port)
                    self.host = new_addr
                    self.family = self._get_inet_family(new_addr)
                    return True
                self._logger('error', 'Cannot guess burp server address')
        return False

    def status(self, query='\n', agent=None):
        """
        status connects to the burp status port, ask the given 'question' and
        parses the output in an array
        """
        r = []
        try:
            if not query.endswith('\n'):
                q = '{0}\n'.format(query)
            else:
                q = query
            s = socket.socket(self.family, socket.SOCK_STREAM)
            s.connect((self.host, self.port))
            s.send(q)
            s.shutdown(socket.SHUT_WR)
            f = s.makefile()
            s.close()
            for l in f.readlines():
                line = l.rstrip('\n')
                if not line:
                    continue
                ap = ''
                try:
                    ap = line.encode('utf-8')
                except UnicodeEncodeError:
                    ap = line
                r.append(ap)
            f.close()
            return r
        except socket.error:
            self._logger('error', 'Cannot contact burp server at %s:%s', self.host, self.port)
            raise BUIserverException('Cannot contact burp server at {0}:{1}'.format(self.host, self.port))

    def get_backup_logs(self, n, c, forward=False, agent=None):
        if not c or not n:
            return []

        f = self.status('c:{0}:b:{1}\n'.format(c, n), agent=agent)
        found = False
        for line in f:
            if line == 'backup_stats':
                found = True
                break

        if not found:
            cl = None
            if forward:
                cl = c

            f = self.status('c:{0}:b:{1}:f:log.gz\n'.format(c, n), agent=agent)
            return self._parse_backup_log(f, n, cl, agent=agent)

        return self._parse_backup_stats(n, c, forward, agent=agent)

    def _parse_backup_stats(self, n, c, forward=False, agent=None):
        backup = { 'windows': 'unknown', 'number': int(n) }
        if forward:
            backup['name'] = c
        keys = {
                'time_start':                    'start',
                'time_end':                      'end',
                'time_taken':                    'duration',
                'bytes_in_backup':               'totsize',
                'bytes_received':                'received',
                'files':                         [ 'files', 'new' ],
                'files_changed':                 [ 'files', 'changed' ],
                'files_same':                    [ 'files', 'unchanged' ],
                'files_deleted':                 [ 'files', 'deleted' ],
                'files_scanned':                 [ 'files', 'scanned' ],
                'files_total':                   [ 'files', 'total' ],
                'files_encrypted':               [ 'files_enc', 'new' ],
                'files_encrypted_changed':       [ 'files_enc', 'changed' ],
                'files_encrypted_same':          [ 'files_enc', 'unchanged' ],
                'files_encrypted_deleted':       [ 'files_enc', 'deleted' ],
                'files_encrypted_scanned':       [ 'files_enc', 'scanned' ],
                'files_encrypted_total':         [ 'files_enc', 'total' ],
                'directories':                   [ 'dir', 'new' ],
                'directories_changed':           [ 'dir', 'changed' ],
                'directories_same':              [ 'dir', 'unchanged' ],
                'directories_deleted':           [ 'dir', 'deleted' ],
                'directories_scanned':           [ 'dir', 'scanned' ],
                'directories_total':             [ 'dir', 'total' ],
                'soft_links':                    [ 'softlink', 'new' ],
                'soft_links_changed':            [ 'softlink', 'changed' ],
                'soft_links_same':               [ 'softlink', 'unchanged' ],
                'soft_links_deleted':            [ 'softlink', 'deleted' ],
                'soft_links_scanned':            [ 'softlink', 'scanned' ],
                'soft_links_total':              [ 'softlink', 'total' ],
                'hard_links':                    [ 'hardlink', 'new' ],
                'hard_links_changed':            [ 'hardlink', 'changed' ],
                'hard_links_same':               [ 'hardlink', 'unchanged' ],
                'hard_links_deleted':            [ 'hardlink', 'deleted' ],
                'hard_links_scanned':            [ 'hardlink', 'scanned' ],
                'hard_links_total':              [ 'hardlink', 'total' ],
                'meta_data':                     [ 'meta', 'new' ],
                'meta_data_changed':             [ 'meta', 'changed' ],
                'meta_data_same':                [ 'meta', 'unchanged' ],
                'meta_data_deleted':             [ 'meta', 'deleted' ],
                'meta_data_scanned':             [ 'meta', 'scanned' ],
                'meta_data_total':               [ 'meta', 'total' ],
                'meta_data_encrypted':           [ 'meta_enc', 'new' ],
                'meta_data_encrypted_changed':   [ 'meta_enc', 'changed' ],
                'meta_data_encrypted_same':      [ 'meta_enc', 'unchanged' ],
                'meta_data_encrypted_deleted':   [ 'meta_enc', 'deleted' ],
                'meta_data_encrypted_scanned':   [ 'meta_enc', 'scanned' ],
                'meta_data_encrypted_total':     [ 'meta_enc', 'total' ],
                'special_files':                 [ 'special', 'new' ],
                'special_files_changed':         [ 'special', 'changed' ],
                'special_files_same':            [ 'special', 'unchanged' ],
                'special_files_deleted':         [ 'special', 'deleted' ],
                'special_files_scanned':         [ 'special', 'scanned' ],
                'special_files_total':           [ 'special', 'total' ],
                'efs_files':                     [ 'efs', 'new' ],
                'efs_files_changed':             [ 'efs', 'changed' ],
                'efs_files_same':                [ 'efs', 'unchanged' ],
                'efs_files_deleted':             [ 'efs', 'deleted' ],
                'efs_files_scanned':             [ 'efs', 'scanned' ],
                'efs_files_total':               [ 'efs', 'total' ],
                'vss_headers':                   [ 'vssheader', 'new' ],
                'vss_headers_changed':           [ 'vssheader', 'changed' ],
                'vss_headers_same':              [ 'vssheader', 'unchanged' ],
                'vss_headers_deleted':           [ 'vssheader', 'deleted' ],
                'vss_headers_scanned':           [ 'vssheader', 'scanned' ],
                'vss_headers_total':             [ 'vssheader', 'total' ],
                'vss_headers_encrypted':         [ 'vssheader_enc', 'new' ],
                'vss_headers_encrypted_changed': [ 'vssheader_enc', 'changed' ],
                'vss_headers_encrypted_same':    [ 'vssheader_enc', 'unchanged' ],
                'vss_headers_encrypted_deleted': [ 'vssheader_enc', 'deleted' ],
                'vss_headers_encrypted_scanned': [ 'vssheader_enc', 'scanned' ],
                'vss_headers_encrypted_total':   [ 'vssheader_enc', 'total' ],
                'vss_footers':                   [ 'vssfooter', 'new' ],
                'vss_footers_changed':           [ 'vssfooter', 'changed' ],
                'vss_footers_same':              [ 'vssfooter', 'unchanged' ],
                'vss_footers_deleted':           [ 'vssfooter', 'deleted' ],
                'vss_footers_scanned':           [ 'vssfooter', 'scanned' ],
                'vss_footers_total':             [ 'vssfooter', 'total' ],
                'vss_footers_encrypted':         [ 'vssfooter_enc', 'new' ],
                'vss_footers_encrypted_changed': [ 'vssfooter_enc', 'changed' ],
                'vss_footers_encrypted_same':    [ 'vssfooter_enc', 'unchanged' ],
                'vss_footers_encrypted_deleted': [ 'vssfooter_enc', 'deleted' ],
                'vss_footers_encrypted_scanned': [ 'vssfooter_enc', 'scanned' ],
                'vss_footers_encrypted_total':   [ 'vssfooter_enc', 'total' ],
                'total':                         [ 'total', 'new' ],
                'total_changed':                 [ 'total', 'changed' ],
                'total_same':                    [ 'total', 'unchanged' ],
                'total_deleted':                 [ 'total', 'deleted' ],
                'total_scanned':                 [ 'total', 'scanned' ],
                'total_total':                   [ 'total', 'total' ]
            }
        f = self.status('c:{0}:b:{1}:f:backup_stats\n'.format(c, n), agent=agent)
        for line in f:
            if line == '-list begin-' or line == '-list end-':
                continue
            (key, val) = line.split(':')
            if backup['windows'] == 'unknown' and key == 'client_is_windows':
                if val == '1':
                    backup['windows'] = 'true'
                else:
                    backup['windows'] = 'false'
                continue
            if not key in keys:
                continue
            rk = keys[key]
            if isinstance(rk, list):
                if not rk[0] in backup:
                    backup[rk[0]] = {}
                backup[rk[0]][rk[1]] = int(val)
            else:
                backup[rk] = int(val)
        return backup

    def _parse_backup_log(self, f, n, c=None, agent=None):
        """
        parse_backup_log parses the log.gz of a given backup and returns a dict
        containing different stats used to render the charts in the reporting view
        """
        lookup_easy = {
                'start':         '^Start time: (.+)$',
                'end':           '^\s*End time: (.+)$',
                'duration':      '^Time taken: (.+)$',
                'totsize':       '^\s*Bytes in backup:\s+(\d+)',
                'received':      '^\s*Bytes received:\s+(\d+)'
                }
        lookup_complex = {
                'files':         '^\s*Files:?\s+(.+)\s+\|\s+(\d+)$',
                'files_enc':     '^\s*Files \(encrypted\):?\s+(.+)\s+\|\s+(\d+)$',
                'dir':           '^\s*Directories:?\s+(.+)\s+\|\s+(\d+)$',
                'softlink':      '^\s*Soft links:?\s+(.+)\s+\|\s+(\d+)$',
                'hardlink':      '^\s*Hard links:?\s+(.+)\s+\|\s+(\d+)$',
                'meta':          '^\s*Meta data:?\s+(.+)\s+\|\s+(\d+)$',
                'meta_enc':      '^\s*Meta data\(enc\):?\s+(.+)\s+\|\s+(\d+)$',
                'special':       '^\s*Special files:?\s+(.+)\s+\|\s+(\d+)$',
                'efs':           '^\s*EFS files:?\s+(.+)\s+\|\s+(\d+)$',
                'vssheader':     '^\s*VSS headers:?\s+(.+)\s+\|\s+(\d+)$',
                'vssheader_enc': '^\s*VSS headers \(enc\):?\s+(.+)\s+\|\s+(\d+)$',
                'vssfooter':     '^\s*VSS footers:?\s+(.+)\s+\|\s+(\d+)$',
                'vssfooter_enc': '^\s*VSS footers \(enc\):?\s+(.+)\s+\|\s+(\d+)$',
                'total':         '^\s*Grand total:?\s+(.+)\s+\|\s+(\d+)$'
                }
        backup = { 'windows': 'false', 'number': int(n) }
        if c is not None:
            backup['name'] = c 
        useful = False
        for line in f:
            if re.match('^\d{4}-\d{2}-\d{2} (\d{2}:){3} \w+\[\d+\] Client is Windows$', line):
                backup['windows'] = 'true'
            elif not useful and not re.match('^-+$', line):
                continue
            elif useful and re.match('^-+$', line):
                useful = False
                continue
            elif re.match('^-+$', line):
                useful = True
                continue

            found = False
            # this method is not optimal, but it is easy to read and to maintain
            for key, regex in lookup_easy.iteritems():
                r = re.search(regex, line)
                if r:
                    found = True
                    if key in ['start', 'end']:
                        backup[key] = int(time.mktime(datetime.datetime.strptime(r.group(1), '%Y-%m-%d %H:%M:%S').timetuple()))
                    elif key == 'duration':
                        tmp = r.group(1).split(':')
                        tmp.reverse()
                        i = 0
                        fields = [0] * 4
                        for v in tmp:
                            fields[i] = int(v)
                            i += 1
                        seconds = 0
                        seconds += fields[0]
                        seconds += fields[1] * 60
                        seconds += fields[2] * (60 * 60)
                        seconds += fields[3] * (60 * 60 * 24)
                        backup[key] = seconds
                    else:
                        backup[key] = int(r.group(1))
                    # break the loop as soon as we find a match
                    break

            # if found is True, we already parsed the line so we can jump to the next one
            if found:
                continue

            for key, regex in lookup_complex.iteritems():
                r = re.search(regex, line)
                if r:
                    self._logger('debug', "match[1]: '{0}'".format(r.group(1)))
                    sp = re.split('\s+', r.group(1))
                    backup[key] = {
                            'new':       int(sp[0]),
                            'changed':   int(sp[1]),
                            'unchanged': int(sp[2]),
                            'deleted':   int(sp[3]),
                            'total':     int(sp[4]),
                            'scanned':   int(r.group(2))
                            }
                    break
        return backup

    def get_counters(self, name=None, agent=None):
        """
        get_counters parses the stats of the live status for a given client and
        returns a dict
        """
        r = {}
        if agent:
            if not name or name not in self.running[agent]:
                return r
        else:
            if not name or name not in self.running:
                return r
        f = self.status('c:{0}\n'.format(name))
        if not f:
            return r
        for line in f:
            self._logger('debug', 'line: {0}'.format(line))
            rs = re.search('^{0}\s+(\d)\s+(\S)\s+(.+)$'.format(name), line)
            if rs and rs.group(2) == 'r' and int(rs.group(1)) == 2:
                c = 0
                for v in rs.group(3).split('\t'):
                    self._logger('debug', '{0}: {1}'.format(self.counters[c], v))
                    if c > 0 and c < 15:
                        val = map(int, v.split('/'))
                        if val[0] > 0 or val[1] > 0 or val[2] or val[3] > 0:
                            r[self.counters[c]] = val
                    else:
                        if 'path' == self.counters[c]:
                            r[self.counters[c]] = v
                        else:
                            r[self.counters[c]] = int(v)
                    c += 1
        if r.viewkeys() & {'start', 'estimated_bytes', 'bytes_in'}:
            diff = time.time() - int(r['start'])
            byteswant = int(r['estimated_bytes'])
            bytesgot = int(r['bytes_in'])
            bytespersec = bytesgot / diff
            bytesleft = byteswant - bytesgot
            r['speed'] = bytespersec
            if (bytespersec > 0):
                timeleft = int(bytesleft / bytespersec)
                r['timeleft'] = timeleft
            else:
                r['timeleft'] = -1
        return r

    def is_backup_running(self, name=None, agent=None):
        """
        is_backup_running returns True if the given client is currently running a
        backup
        """
        if not name:
            return False
        try:
            f = self.status('c:{0}\n'.format(name))
        except BUIserverException:
            return False
        for line in f:
            r = re.search('^{0}\s+\d\s+(\w)'.format(name), line)
            if r and r.group(1) not in [ 'i', 'c', 'C' ]:
                return True
        return False

    def is_one_backup_running(self, agent=None):
        """
        is_one_backup_running returns a list of clients name that are currently
        running a backup
        """
        r = []
        try:
            cls = self.get_all_clients()
        except BUIserverException:
            return r
        for c in cls:
            if self.is_backup_running(c['name'], agent):
                r.append(c['name'])
        self.running = r
        return r

    def get_all_clients(self, agent=None):
        """
        get_all_clients returns a list of dict representing each clients with their
        name, state and last backup date
        """
        j = []
        f = self.status()
        for line in f:
            regex = re.compile('\s*(\S+)\s+\d\s+(\S)\s+(.+)')
            m = regex.search(line)
            c = {}
            c['name'] = m.group(1)
            c['state'] = self.states[m.group(2)]
            infos = m.group(3)
            if c['state'] in ['running']:
                c['last'] = 'now'
            elif infos == "0":
                c['last'] = 'never'
            elif re.match('^\d+\s\d+\s\d+$', infos):
                sp = infos.split()
                c['last'] = datetime.datetime.fromtimestamp(int(sp[2])).strftime('%Y-%m-%d %H:%M:%S')
            else:
                sp = infos.split('\t')
                c['last'] = datetime.datetime.fromtimestamp(int(sp[len(sp)-2])).strftime('%Y-%m-%d %H:%M:%S')
            j.append(c)
        return j

    def get_client(self, name=None, agent=None):
        """
        get_client returns a list of dict representing the backups (with its number
        and date) of a given client
        """
        r = []
        if not name:
            return r
        c = name
        f = self.status('c:{0}\n'.format(c))
        for line in f:
            if not re.match('^{0}\t'.format(c), line):
                continue
            self._logger('debug', "line: '{0}'".format(line))
            regex = re.compile('\s*(\S+)\s+\d\s+(\S)\s+(.+)')
            m = regex.search(line)
            if m.group(3) == "0" or m.group(2) not in [ 'i', 'c', 'C' ]:
                continue
            backups = m.group(3).split('\t')
            for b in backups:
                ba = {}
                sp = b.split()
                ba['number'] = sp[0]
                ba['date'] = datetime.datetime.fromtimestamp(int(sp[2])).strftime('%Y-%m-%d %H:%M:%S')
                r.append(ba)
        # Here we need to reverse the array so the backups are sorted by date ASC
        r.reverse()
        return r

    def get_tree(self, name=None, backup=None, root=None, agent=None):
        """
        get_tree returns a list of dict representing files/dir (with their attr)
        within a given path
        """
        r = []
        if not name or not backup:
            return r
        if not root:
            top = ''
        else:
            try:
                top = root.encode('utf-8')
            except UnicodeEncodeError:
                top = root

        f = self.status('c:{0}:b:{1}:p:{2}\n'.format(name, backup, top))
        useful = False
        for line in f:
            self._logger('debug', "line: '{0}'".format(line))
            if not useful and re.match('^-list begin-$', line):
                useful = True
                continue
            if useful and re.match('^-list end-$', line):
                useful = False
                continue
            if useful:
                t = {}
                m = re.search('^(.{10})\s', line)
                if m:
                    if re.match('^d', m.group(1)):
                        t['type'] = 'd'
                    else:
                        t['type'] = 'f'
                    sp = re.split('\s+', line, 7)
                    t['mode'] = sp[0]
                    t['inodes'] = sp[1]
                    t['uid'] = sp[2]
                    t['gid'] = sp[3]
                    t['size'] = '{0:.1eM}'.format(_hr(sp[4]))
                    t['date'] = '{0} {1}'.format(sp[5], sp[6])
                    t['name'] = sp[7]
                    t['parent'] = top
                    r.append(t)
        return r

    def restore_files(self, name=None, backup=None, files=None, strip=None, agent=None):
        if not name or not backup or not files or not self.stripbin or not self.burpbin:
            return None
        flist = json.loads(files)
        if 'restore' not in flist:
            return None
        if os.path.isdir(self.tmpdir):
            shutil.rmtree(self.tmpdir)
        full_reg = u''
        for r in flist['restore']:
            reg = u''
            if r['folder'] and r['key'] != '/':
                reg += '^'+re.escape(r['key'])+'/|'
            else:
                reg += '^'+re.escape(r['key'])+'$|'
            full_reg += reg

        cmd = [self.burpbin, '-C', name, '-a', 'r', '-b', str(backup), '-r', full_reg.rstrip('|'), '-d', self.tmpdir]
        if self.burpconfcli:
            cmd.append('-c')
            cmd.append(self.burpconfcli)
        if strip and strip.isdigit() and int(strip) > 0:
            cmd.append('-s')
            cmd.append(strip)
        self._logger('debug', cmd)
        status = subprocess.call(cmd)
        self._logger('debug', 'command returned: %d', status)
        # a return code of 2 means there were some warnings during restoration
        # so we can assume the restoration was successful anyway
        if status not in [0, 2]:
            return None

        zip_dir = self.tmpdir.rstrip(os.sep)
        zip_file = zip_dir+'.zip'
        if os.path.isfile(zip_file):
            os.remove(zip_file)
        zip_len = len(zip_dir) + 1
        stripping = True
        with zipfile.ZipFile(zip_file, mode='w', compression=zipfile.ZIP_DEFLATED) as zf:
            for dirname, subdirs, files in os.walk(zip_dir):
                for filename in files:
                    path = os.path.join(dirname, filename)
                    if stripping and os.path.isfile(path):
                        self._logger('debug', "stripping file: %s", path)
                        shutil.move(path, path+'.tmp')
                        status = subprocess.call([self.stripbin, '-i', path+'.tmp', '-o', path])
                        if status != 0:
                            os.remove(path)
                            shutil.move(path+'.tmp', path)
                            stripping = False
                            self._logger('debug', "Disable stripping since this file does not seem to embed VSS headers")
                        else:
                            os.remove(path+'.tmp')

                    entry = path[zip_len:]
                    zf.write(path, entry)

        shutil.rmtree(self.tmpdir)
        return zip_file

    def read_conf(self, agent=None):
        if not self.parser:
            return []
        return self.parser.read_server_conf()

    def store_conf(self, data, agent=None):
        if not self.parser:
            return []
        return self.parser.store_server_conf(data)

    def get_parser_attr(self, attr=None, agent=None):
        if not attr or not self.parser:
            return None
        return self.parser.get_priv_attr(attr)



# -*- coding: utf8 -*-

class BUIbackend:
    def __init__(self, app=None, host='127.0.0.1', port=4972):
        self.app = app
        self.host = host
        self.port = port

    def status(self, query='\n', agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_backup_logs(self, n, c, forward=False, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_counters(self, name=None, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def is_backup_running(self, name=None, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def is_one_backup_running(self, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_all_clients(self, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_client(self, name=None, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_tree(self, name=None, backup=None, root=None, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def restore_files(self, name=None, backup=None, files=None, strip=None, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def read_conf(self, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def store_conf(self, data, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

    def get_parser_attr(self, attr=None, agent=None):
        raise NotImplementedError("Sorry, the current Backend does not implement this method!")

class BUIserverException(Exception):
    pass

class BUIserverException(Exception):
    pass



#!/usr/bin/env python
# coding: utf-8

import os
import os.path
import re
import sys

from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'burpui', '__init__.py')) as f:
    data = f.read()

    name = re.search("__title__ *= *'(.*)'", data).group(1)
    author = re.search("__author__ *= *'(.*)'", data).group(1)
    author_email = re.search("__author_email__ *= *'(.*)'", data).group(1)
    description = re.search("__description__ *= *'(.*)'", data).group(1)
    url = re.search("__url__ *= *'(.*)'", data).group(1)

with open('requirements.txt', 'r') as f:
    requires = [x.strip() for x in f if x.strip()]

with open('test-requirements.txt', 'r') as f:
    test_requires = [x.strip() for x in f if x.strip()]

datadir = os.path.join('share', 'burpui', 'etc')

setup(
    name=name,
    version=open('VERSION').read().rstrip(),
    description=description,
    long_description=open('README.rst').read(),
    license=open('LICENSE').read(),
    author=author,
    author_email=author_email,
    url=url,
    keywords='burp web ui',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'static': 'burpui/static/*',
        'templates': 'burpui/templates/*'
    },
    scripts=['bin/burp-ui', 'bin/bui-agent'],
    data_files=[(datadir, [os.path.join(datadir, 'burpui.cfg')]),
                (datadir, [os.path.join(datadir, 'buiagent.cfg')])
    ],
    install_requires=requires,
    extras_require={
        'ldap_authentication': ['simpleldap==0.8']
    },
    tests_require=test_requires,
    classifiers=[
        'Framework :: Flask',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: System :: Archiving :: Backup',
        'Topic :: System :: Monitoring'
    ]
)
